/*
 *
 * B2IVRInterface.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: B2BIVRInterface.h,v $
 * Revision 1.7  2007/07/14 07:50:15  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.6  2007/07/03 17:17:33  joegenbaclor
 * Got rid of compile warnings in unix builds
 *
 * Revision 1.5  2007/06/15 09:12:14  joegenbaclor
 * Added IVR Transfer reject event
 *
 * Revision 1.4  2007/06/12 09:22:38  joegenbaclor
 * Added IVR Call Transfer
 *
 * Revision 1.3  2007/06/11 23:51:45  joegenbaclor
 * Added event queue
 *
 * Revision 1.2  2007/06/08 15:10:25  joegenbaclor
 * Added IVR DTMF Grammar
 *
 * Revision 1.1  2007/06/08 10:32:31  joegenbaclor
 * Initial upload
 *
 *
 */

#ifndef B2BIVRINTERFACE_H
#define B2BIVRINTERFACE_H

#include "B2BInterface.h"
#include "EventQueue.h"
#include "SIPSession.h"

using namespace EQ;
using namespace UACORE;


namespace B2BUA
{
  class B2BUAConnection;

  class B2BIVRInterface : public B2BInterface
  {
    PCLASSINFO( B2BIVRInterface, B2BInterface );
  public:

    class B2BIVREvent : public PObject
    {
    public:
      enum Type
      {
        Event_IVROnOpenChannel,
        Event_IVROnEndPlaying,
        Event_IVROnUserInput,
        Event_IVROnEndCollection,
        Event_IVROnTransferReject
      };

      B2BIVREvent( 
        B2BUAConnection & connection,
        Type type 
      );

      PINLINE Type GetType()const{ return m_Type; };
      PINLINE SIPSession::GCRef & GetConnection(){ return m_Connection; };

    protected:

      Type m_Type;
      SIPSession::GCRef m_Connection;
    };

    class OnOpenChannel : public B2BIVRInterface::B2BIVREvent
    {
    public:
      OnOpenChannel( 
        B2BUAConnection & connection
        ) : B2BIVREvent( connection, B2BIVREvent::Event_IVROnOpenChannel )
      {
      }
    };


    class OnEndPlaying : public B2BIVRInterface::B2BIVREvent
    {
    public:
      OnEndPlaying( 
        B2BUAConnection & connection,
        const OString & identifier
        ) : B2BIVREvent( connection, B2BIVREvent::Event_IVROnEndPlaying )
      {
        m_Identifier = identifier;
      }
      OString m_Identifier;
    };

    class OnUserInput : public B2BIVRInterface::B2BIVREvent
    {
    public:
      OnUserInput( 
        B2BUAConnection & connection,
        const OString & userInput
        ) : B2BIVREvent( connection, B2BIVREvent::Event_IVROnUserInput )
      {
        m_UserInput = userInput;
      }
      OString m_UserInput;
    };

    class OnEndCollection : public B2BIVRInterface::B2BIVREvent
    {
    public:
      OnEndCollection(
        B2BUAConnection & connection,
        const OString &grammarId, 
        const OString &grammarBuffer 
      ) : B2BIVREvent( connection, B2BIVREvent::Event_IVROnEndCollection )
      {
        m_GrammarId = grammarId;
        m_GrammarBuffer = grammarBuffer;
      }

      OString m_GrammarId;
      OString m_GrammarBuffer;
    };

    class OnTransferReject : public B2BIVRInterface::B2BIVREvent
    {
    public:
      OnTransferReject(
        B2BUAConnection & connection,
        const SIPMessage & reject
      ) : B2BIVREvent( connection, B2BIVREvent::Event_IVROnTransferReject )
      {
        m_RejectMsg = reject;
      }

      SIPMessage m_RejectMsg;
    };

    B2BIVRInterface(  
      B2BUserAgent & b2bua 
    );

    virtual ~B2BIVRInterface();

    virtual void OnIVREvent(
      B2BUAConnection * conn,
      const B2BIVREvent * evt
    );


  protected:

    

    virtual BOOL IVROnOpenChannel(
      B2BUAConnection & connection
    );

    virtual void IVROnEndPlaying(
      B2BUAConnection & connection,
      const OString & identifier
    );

    virtual BOOL IVROnUserInput(
      B2BUAConnection & connection,
      const OString & str 
    );

    virtual void IVROnEndCollection( 
      B2BUAConnection & connection,
      const OString &grammarId, 
      const OString &grammarBuffer 
    );

    virtual void IVROnTransferReject(
      B2BUAConnection & connection,
	    const SIPMessage & reject
	  );


    void EnqueueEvent( B2BIVREvent * event );

    PDECLARE_NOTIFIER( EventQueue, B2BIVRInterface, _OnIVREvent );
    PMutex m_EventQueueMutex;
    EventQueue * m_EventQueue;

    friend class B2BUserAgent;
  };
};

#endif //B2BIVRINTERFACE_H

