/*
 *
 * B2BRoutingInterface.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: B2BRoutingInterface.h,v $
 * Revision 1.15  2007/07/14 07:50:15  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.14  2007/07/07 15:19:13  joegenbaclor
 * Added iproute2 support for linux
 *
 * Revision 1.13  2007/06/29 08:21:59  joegenbaclor
 * Removed constness for request param in B2BRouteRequest and RouteLocalRegistration to allow the target address to be set early for the request
 *
 * Revision 1.12  2007/06/18 15:30:01  joegenbaclor
 * Implemented Internal DNS Mapping
 * Fixed bug in Cache Manager
 *
 * Revision 1.11  2007/06/17 02:49:14  joegenbaclor
 * Introduced Global lock to PHTTPConfig
 *
 * Revision 1.10  2007/06/16 12:30:35  joegenbaclor
 * Changed CacheManager to return a smart pointer instead of a direct pointer to cache object to avoid segmentation faults when an object expires in cache and another thread has pending ownership of the pointer
 *
 * Renamed CacheManager::GetAt() to CacheManager::CreatePointer() to give a hint to users that the pointer needs to be deleted after being used
 *
 * Revision 1.9  2007/05/18 01:24:44  joegenbaclor
 * 1.  Added separate useragent to handle B2BUpperReg backdoor
 * 2.  Modified routing for B2BUpperReg
 * 3.  Corrected bug in ParserTools::GetCompactHeader()
 *
 * Revision 1.8  2007/02/16 11:09:24  joegenbaclor
 * More work on privacy
 *
 * Revision 1.7  2006/11/30 08:19:35  joegenbaclor
 * Relicensed opensipstack to MPL/GPL/LGPL triple license scheme
 *
 * Revision 1.6  2006/11/23 11:00:02  joegenbaclor
 * Added Instant Message B2B support
 *
 * Revision 1.5  2006/11/23 01:28:18  joegenbaclor
 * Corrected compile errors from ryan's last commit
 *
 * Revision 1.4  2006/07/11 13:59:35  joegenbaclor
 * Completed Proxy functionalities for B2BUA
 *
 * Revision 1.3  2006/07/03 15:29:47  joegenbaclor
 * Started merging OpenB2BUA into OpenSBC
 *
 * Revision 1.2  2006/06/21 14:29:11  joegenbaclor
 * This upload marks the first successful call by OpenB2BUA
 *
 * Revision 1.1  2006/06/15 09:47:28  joegenbaclor
 * More preliminary design work on B2BUA
 *
 *
 */
#ifndef B2BROUTINGINTERFACE_H
#define B2BROUTINGINTERFACE_H

#include "B2BInterface.h"
#include "ProxySession.h"

using namespace UACORE;
using namespace SIPTransports;

#ifndef CALL_TRANSFER_PREFIX
#define CALL_TRANSFER_PREFIX "xfer-oss-"
#endif

namespace B2BUA
{

  class B2BRoutingInterface : public B2BInterface, public SIPTransportManager::ExternalRouteTableAnalyzer
  {
    PCLASSINFO( B2BRoutingInterface, B2BInterface );
  public:

    class Resolver : public SIPSrvRecord::Resolver
    {
    public:
      Resolver( 
        B2BRoutingInterface * iface 
      )
      {
        m_Iface = iface;
      }

      virtual BOOL Resolve(
        const OString & domain,
        OString & result
      );

      B2BRoutingInterface * m_Iface;
    };

    B2BRoutingInterface(  
      B2BUserAgent & b2bua 
    );

    virtual ~B2BRoutingInterface();

    virtual BOOL B2BRouteCall(
      B2BUAConnection & connection,
      SIPMessage & invite,
      BOOL ignoreRegistrations = FALSE
    );

    virtual BOOL B2BRouteRequest(
      SIPMessage & request,
      SIPURI  & route
    );

    virtual ProxySession::RoutePolicy RouteProxyRequest(
      ProxySession & session,
      SIPMessage & request
    );

    virtual void OnRequireRedirectionId(
      B2BUAConnection & connection,
      OString & redirectionId
    );

    virtual void OnRequireCallTransferId(
      B2BUAConnection & connection,
      const ReferTo & referTo, 
      OString & redirectionId
    );

    virtual void OnOrphanedMessage(
      SIPMessage & message
    );

    /// allows implementors to have a custom resolution routine for DNS
    virtual BOOL OnInternalResolveAddress(
      const OString & domain,
      OString & result
    );
    
    virtual BOOL OnGetRouteTableEntry(
      const PIPSocket::Address & destination,
      PIPSocket::Address & source,
      PIPSocket::Address & route,
      OString & device 
    );

    PLIST( B2BRouteList, SIPURI );

  protected:
    Resolver * m_Resolver;
  };
}

#endif

