/*
 *
 * B2BUAConnection.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: B2BUAConnection.h,v $
 * Revision 1.55  2007/07/26 08:15:51  joegenbaclor
 * Uploading support for AuthenticationPending state in B2BUAConnection.  This fix will eliminatethe need  to generate a diffrent call-id for resending credentials for 401/407
 *
 * Revision 1.54  2007/07/15 13:33:10  joegenbaclor
 * deprecated old connection count variable
 *
 * Revision 1.53  2007/07/14 07:50:16  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.52  2007/07/06 13:38:39  rcolobong
 * Add Upstream Proxy Route feature
 *
 * Revision 1.51  2007/06/16 12:30:35  joegenbaclor
 * Changed CacheManager to return a smart pointer instead of a direct pointer to cache object to avoid segmentation faults when an object expires in cache and another thread has pending ownership of the pointer
 *
 * Renamed CacheManager::GetAt() to CacheManager::CreatePointer() to give a hint to users that the pointer needs to be deleted after being used
 *
 * Revision 1.50  2007/06/15 09:12:14  joegenbaclor
 * Added IVR Transfer reject event
 *
 * Revision 1.49  2007/06/12 09:22:38  joegenbaclor
 * Added IVR Call Transfer
 *
 * Revision 1.48  2007/06/08 15:10:28  joegenbaclor
 * Added IVR DTMF Grammar
 *
 * Revision 1.47  2007/06/08 02:33:45  joegenbaclor
 * Commiting more Media Server Related Changes
 *
 * Revision 1.46  2007/06/06 09:54:05  joegenbaclor
 * More B2BUAConnection IVR methods
 *
 * Revision 1.45  2007/06/05 16:27:54  joegenbaclor
 * Added IVR audio playback functions to B2BUAConnection
 *
 * Revision 1.44  2007/05/27 06:15:49  joegenbaclor
 * Implemented OnReceivedSDPAnswer()
 *
 * Revision 1.43  2007/05/16 08:11:14  rcolobong
 * 1. Bug fix on Alerting and Seize Timeout
 * 2. Bug fix on AttachCall method where existing B2BUACall is overriden without releasing its reference
 *
 * Revision 1.42  2007/05/08 05:03:24  joegenbaclor
 * Added 491 Request Pending Support
 *
 * Revision 1.41  2007/05/06 15:52:14  joegenbaclor
 * More bug fixes in Delayed Deletion
 *
 * Revision 1.40  2007/05/04 08:44:39  joegenbaclor
 * Introduced delayed deletion mechanism
 *
 * Revision 1.39  2007/05/01 04:19:36  rcolobong
 * 1. Add support for enabling and disabling Local Refer
 * 2. Invoke OnCallRejected during failover
 * 3. Add method GenerateCallIdExtensions
 *
 * Revision 1.38  2007/04/17 08:02:06  joegenbaclor
 * more work on local refer
 *
 * Revision 1.37  2007/04/16 14:24:39  joegenbaclor
 * More work on B2B local REFER support
 *
 * Revision 1.36  2007/02/16 11:09:24  joegenbaclor
 * More work on privacy
 *
 * Revision 1.35  2007/02/09 03:47:33  joegenbaclor
 * 1. Added MaxForwards handling for B2B calls
 * 2. #defined SIP_DEFAULT_MAX_FORWARDS constant
 *
 * Revision 1.34  2007/02/08 18:05:08  joegenbaclor
 * Corrected another bug in Record routing
 *
 * Revision 1.33  2007/01/10 08:07:15  rcolobong
 * 1. Add new method for OnSendB2BConnect Call Interface.
 * 2. OnSendB2BConnect is supported in B2BUserAgent, B2BUACall and B2BUAConnection
 *
 * Revision 1.32  2006/12/19 10:14:26  rcolobong
 * 1. Add method to insert Warning header in 200 Ok response to inbound
 * 2. Alerting timeout will now do failover
 *
 * Revision 1.31  2006/12/17 13:30:52  joegenbaclor
 * Added call attempt index in call-id string to avoid call-id collision during failovers
 *
 * Revision 1.30  2006/11/30 08:19:35  joegenbaclor
 * Relicensed opensipstack to MPL/GPL/LGPL triple license scheme
 *
 * Revision 1.29  2006/11/10 07:55:35  rcolobong
 * 1. Support Fork Invites using B2BUAForkCalls
 * 2. Handle Cancel events during Forking
 *
 * Revision 1.28  2006/10/30 08:35:55  joegenbaclor
 * Removed more references to presence namespace
 *
 * Revision 1.27  2006/10/23 15:44:54  joegenbaclor
 * 1.  Numerous bug fixes for REFER
 * 2.  Added new SIP Headers for SIP timers and Attended call transfer
 * 3.  Added loop sanity check to SIP UDP Transport
 * 4.  Corrected parser bug for ProxyRequire header
 *
 * Revision 1.26  2006/10/11 04:49:47  rcolobong
 * 1. Add support for Notify During Refer
 * 2. Fixed bug where 3xx invoke CallHandler OnRejected
 *
 * Revision 1.25  2006/09/25 04:43:29  rcolobong
 * 1. Fix bug in 415 Unsupported Media where CallSession::m_ICTState is not properly initialized during construction.
 * 2. Remove the temporary fix for the 415 Unsupported Media where we use Semaphore to wait for second leg SDP in OnRequireSDPAnswer
 *
 * Revision 1.24  2006/09/22 08:52:45  joegenbaclor
 * Temporary fix for OnRequireSDPAnswer to use a semaphore to wait for second leg SDP
 *
 * Revision 1.23  2006/09/20 11:05:25  joegenbaclor
 * Flagged sending of Session Keep Alive.  Defaults to FALSE
 *
 * Revision 1.22  2006/09/18 09:35:15  joegenbaclor
 * Completed initial implementation of RTP media aggregation
 *
 * Revision 1.21  2006/09/18 03:28:54  joegenbaclor
 * 1. More work on media aggregation
 * 2. Check sock->GetPort() for NULL in SIPTransportManager::GetListenerAddress() (thanks to
 *  Vamsi Pottangi for reporting the bug)
 * 3.  Added reference size check to GCObject::ReleaseRef()
 *
 * Revision 1.20  2006/09/15 12:29:44  joegenbaclor
 * Flagged OnDestroySession of B2BConnection so it wont be called twice from different
 *  threads.
 *
 * Revision 1.19  2006/09/08 04:39:16  joegenbaclor
 * Implemented INFO Session Keep-alive
 *
 * Revision 1.18  2006/09/01 13:08:32  rcolobong
 * Determining if we would enable Media Proxy is now set in B2BUAConnection
 *
 * Revision 1.17  2006/08/30 08:42:10  rcolobong
 * 1. Update handling for connection and alerting timeout
 * 2. Fix bug where leg1 is destroyed before calling DestroyConnection
 *
 * Revision 1.16  2006/08/29 10:46:05  rcolobong
 * *** empty log message ***
 *
 * Revision 1.15  2006/08/26 14:31:40  joegenbaclor
 * Bulk Commit:  All SIP Sessions are now using new Garbage Collector and smart Pointers
 *
 * Revision 1.14  2006/08/04 05:36:49  rcolobong
 * Add Alerting and Connecting Timer
 *
 * Revision 1.13  2006/07/19 02:19:20  joegenbaclor
 * 1.  Bug fixes concerning B2BUA destruction during during 3xx, 4xx, 5xx, 6xx local response
 * 2.  Fixed bug where Route header is never removed from outbound Invite resulting that may result to a loop when
 *      Inbound Invite has a Route header present
 * 3.  Added RemoveAllRoutes() in SIPMessage
 *
 * Revision 1.12  2006/07/13 06:53:04  joegenbaclor
 * 1.  Introduced Sanity checks for incoming SIP Message
 * 2.  Corrected bug in SIPURI where enum scheme is not getting recognized.
 * 3.  Added ENUM support to routing
 * 4.  Introduced global routing flag to indicate when and not to rewrite outbound To URI's
 *
 * Revision 1.11  2006/06/29 05:09:13  joegenbaclor
 * Changed OnOutgoingCall invite parameter from const to none const to give a
 * chance for applications to modify the outbound invite before being sent to the transport
 *
 * Revision 1.10  2006/06/28 14:14:56  joegenbaclor
 * Bulk commit for Media Proxy functionalities for OpenB2BUA
 * Marks a minor version upgrade from 1.0.2 to 1.0.3
 *
 * Revision 1.9  2006/06/23 04:44:04  joegenbaclor
 * SendB2BRefer() added to B2BUACall
 *
 * Revision 1.8  2006/06/22 08:27:56  joegenbaclor
 * *** empty log message ***
 *
 * Revision 1.7  2006/06/21 14:29:11  joegenbaclor
 * This upload marks the first successful call by OpenB2BUA
 *
 * Revision 1.6  2006/06/20 09:52:55  joegenbaclor
 * Added Routing and SDP Handlers to B2BUA
 *
 * Revision 1.5  2006/06/15 09:47:27  joegenbaclor
 * More preliminary design work on B2BUA
 *
 * Revision 1.4  2006/06/14 08:43:38  joegenbaclor
 * Initial upload of OpenB2BUA applcation and related classes
 *
 * Revision 1.3  2006/04/11 00:18:54  joegenbaclor
 * More work on B2BUA class structures
 *
 * Revision 1.2  2006/04/10 01:09:03  joegenbaclor
 * Implemented Create methods fo B2BUA sessions
 *
 * Revision 1.1  2006/04/08 06:27:13  joegenbaclor
 * Initial upload of B2BUA classes
 *
 *
 */

#ifndef B2BUACONNECTION_H
#define B2BUACONNECTION_H

#include "B2BUACall.h"
#include "B2BUA.h"
#include "MediaServerSession.h"

namespace B2BUA
{
  class B2BUAForkCalls;

  class B2BUAConnection : public SIPSession
  {
    PCLASSINFO( B2BUAConnection, SIPSession );

  public:
    enum SessionEvents
    {
      SetupOutbound,
      DestroySession,
      ErrorAlertingTimeout,
      ErrorSeizeTimeout,
      ErrorNoRoute,
      ErrorRouteResolution
    };


    enum SessionState
    {
      Idle = 0,
      Trying,
      Proceeding,
      Transferring,
      AuthenticationPending,
      Connected,
      Disconnected,
      Terminated
    };

    B2BUAConnection(
      B2BUAEndPoint & ep,
      const OString & sessionId
    );

    virtual ~B2BUAConnection();

    virtual void OnDestroySession();

    virtual BOOL OnIncomingCall(
      B2BUACall & call,
      SIPMessage & invite
    );

    virtual void OnRequireRedirectionId(
      B2BUACall & call,
      OString & redirectionId
    );

    virtual void OnRequireCallTransferId(
      B2BUACall & call,
      const ReferTo & referTo, 
      OString & redirectionId
    );

    virtual BOOL OnIncomingSDPOffer(
      B2BUACall & call,
      const SIPMessage & sdp
    );

    virtual BOOL OnIncomingSDPAnswer(
      B2BUACall & call,
      const SIPMessage & sdp
    );

    /// your last chance to produce an answer to the offer.  
    /// This will be called either before sending 183, 200 or ACK 
    virtual BOOL OnRequireSDPAnswer(
      B2BUACall & call,
      const SIPMessage & offer,
      SIPMessage & answer
    );

    /// this will be called if an offer is not received in INVITE or when we are initiating the call
    virtual BOOL OnRequireSDPOffer(
      B2BUACall & call,
      SIPMessage & offer
    );

    virtual BOOL OnSendB2BConnect(
      B2BUACall & call,
      const SIPMessage & message
    );

    BOOL AttachCall(
      const OString & id,
      B2BUACall * call,
      int index = -1
    );

    SIPSession::GCRef * DetachCall(
      const OString & id
    );

    SIPSession::GCRef * FindCall(
      const OString & id
    );

    virtual void OnSessionEvent(
      int event,
      const SIPMessage & eventMsg
    );

    virtual void OnSetupOutbound(
      const SIPMessage & inboundInvite
    );

    virtual void OnOutgoingCall(
      B2BUACall & call,
      SIPMessage & outboundInvite );


    virtual void OnHandleLocalRefer( 
      B2BUACall & call,
      SIPMessage & refer 
    );
    
    void SetSessionState(
      B2BUACall & call,
      const SIPMessage & msg
    );

    /*
    void SetSessionState(
      B2BUACall & call,
      int event
    );
    */

    void OnSessionKeepAliveTimeout(
      B2BUACall & call 
    );

    void OnHandlePreConnectState( 
      B2BUACall & call,
      const SIPMessage & msg
    );

    void OnHandleStateTransferring( 
      B2BUACall & call,
      const SIPMessage & msg
    );

    void OnHandleStateAuthenticationPending( 
      B2BUACall & call,
      const SIPMessage & msg
    );
    
    void OnHandleStateConnected( 
      B2BUACall & call,
      const SIPMessage & msg
    );

    void OnHandleStateDisconnected( 
      B2BUACall & call,
      const SIPMessage & msg
    );

    void OnHandleStateTerminated( 
      B2BUACall & call,
      const SIPMessage & msg
    );

    void DestroyConnection( 
      const SIPMessage & msg /// SIP Message that caused the session to be destroyed
    );

    void DestroyConnection();
    
    virtual void OnSelfDestruct();

    virtual void OnGCCollect();

    PINLINE void Post200OkWarningMessage( const Warning& warning ) { m_200OkWarning = warning; }

    PINLINE void EnableForkCalls( BOOL enable = TRUE ) { m_EnableForkCalls = enable; }
    PINLINE BOOL IsForkCalls() const { return m_EnableForkCalls; }
    PINLINE BOOL IsUpstreamProxy() const { return m_EnableUpstreamProxyRoute; }
    PINLINE B2BUAForkCalls * GetForkCalls() { return m_ForkCalls; }

    PINLINE void AddRoute( const SIPURI & route ){ m_Routes.Append( new SIPURI( route ) ); };
    PINLINE void RemoveRoutes(){ m_Routes.RemoveAll(); };
    PINLINE void AddUpstreamProxyRoute( const SIPURI & route ){ m_UpstreamProxyRoutes.Append( new SIPURI( route ) ); };
    PINLINE void RemoveUpstreamProxyRoutes(){ m_UpstreamProxyRoutes.RemoveAll(); };
    PINLINE void SetUpstreamBackdoorURI( const OString & uri ) { m_UpstreamBackdoorURI = uri; }
    PINLINE const OString & GetUpstreamBackdoorURI() const { return m_UpstreamBackdoorURI; }
    PINLINE B2BRoutingInterface::B2BRouteList & GetRoutes(){ return m_Routes; };
    PINLINE B2BRoutingInterface::B2BRouteList & GetUpstreamProxyRoutes(){ return m_UpstreamProxyRoutes; }
    PINLINE const OString & GetLeg1SDP()const{ return m_Leg1SDP; };
    PINLINE void SetLeg1SDP( const OString & sdp ){ m_Leg1SDP = sdp; };
    PINLINE const OString & GetLeg2SDP()const{ return m_Leg2SDP; };
    PINLINE void SetLeg2SDP( const OString & sdp ){ m_Leg2SDP = sdp; };
    PINLINE B2BUACall * GetLeg1Call(){ return m_Leg1Call; };
    PINLINE B2BUACall * GetLeg2Call(){ return m_Leg2Call; };
    PINLINE void SetMediaProxyIfPrivate( BOOL media = TRUE ) { m_MediaProxyIfPrivate = media; }
    PINLINE BOOL GetMediaProxyIfPrivate() const { return m_MediaProxyIfPrivate; }
    PINLINE void EnableLocalRefer( BOOL enable = TRUE ) { m_EnableLocalRefer = enable; }
    PINLINE void DisableLocalRefer() { m_EnableLocalRefer = FALSE; }
    PINLINE void EnableUpstreamProxyRoute( BOOL enable = TRUE ) { m_EnableUpstreamProxyRoute = enable; }
    PINLINE void DisableUpstreamProxyRoute() { m_EnableUpstreamProxyRoute = FALSE; }
    PINLINE SessionState GetSessionState()const{ return m_SessionState; };
    PINLINE PINDEX GetCallTransferIndex()const{ return m_CallTransferIndex; };
    static void SetSeizeTimeout( DWORD timeout ){ m_SeizeTimeout = timeout; }
    static void SetAlertingTimeout( DWORD timeout ){ m_AlertingTimeout = timeout; }

  protected:
    PMutex m_CallSessionsMutex;
    PDICTIONARY( CallSessionList, PCaselessString, SIPSession::GCRef );
    CallSessionList m_CallSessions;
    B2BUACall * m_Leg1Call;
    B2BUACall * m_Leg2Call;
    B2BUAForkCalls * m_ForkCalls;
    PINDEX m_CallTransferIndex;
    BOOL m_ICTRequestPending;
    static DWORD m_SeizeTimeout;
    static DWORD m_AlertingTimeout;
    BOOL m_MediaProxyIfPrivate;
    BOOL m_EnableForkCalls;
    BOOL m_EnableUpstreamProxyRoute;

    B2BRoutingInterface::B2BRouteList m_Routes;
    B2BRoutingInterface::B2BRouteList m_UpstreamProxyRoutes;
    OString m_UpstreamBackdoorURI;

    OString m_Leg1SDP;
    OString m_Leg2SDP;

    PMutex m_SessionStateMutex;
    SessionState m_SessionState;

    Warning m_200OkWarning;

    PTimer m_SeizeTimer;
    PDECLARE_NOTIFIER( PTimer, B2BUAConnection, OnSeizeTimeout );

    PTimer m_AlertingTimer;
    PDECLARE_NOTIFIER( PTimer, B2BUAConnection, OnAlertingTimeout );

  protected:
    void StartSeizeTimer();
    void StopSeizeTimer();
    void StartAlertingTimer();
    void StopAlertingTimer();


    /** IST Callbacks */
    void IST_OnReceivedMoreInvite(
      B2BUACall & call,
      const SIPMessage & msg
    );

    /** NIST Callbacks */
    virtual void NIST_OnReceivedRegister(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedRefer(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedBye(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedOptions(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedInfo(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedMessage(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedCancel(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedNotify(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedSubscribe(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NIST_OnReceivedUnknown(
      B2BUACall & call,
      const SIPMessage & msg
    );

  /** NICT CALLBACKS */
    virtual void NICT_OnReceived1xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NICT_OnReceived2xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NICT_OnReceived3xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NICT_OnReceived4xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NICT_OnReceived5xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NICT_OnReceived6xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void NICT_OnReceivedUnknown(
      B2BUACall & call,
      const SIPMessage & msg
    );

  /** ICT CALLBACKS */
    virtual void ICT_OnReceived1xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void ICT_OnReceived2xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void ICT_OnReceived3xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void ICT_OnReceived4xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void ICT_OnReceived5xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void ICT_OnReceived6xx(
      B2BUACall & call,
      const SIPMessage & msg
    );

    virtual void ICT_OnReceivedUnknown(
      B2BUACall & call,
      const SIPMessage & msg
    );

    friend class B2BUACall;

#if HAS_OPAL
  public:
    PINLINE BOOL HasStartedMediaStream()const{ return m_HasStartedMediaStream; };
    PINLINE void StartMediaStream(){ m_HasStartedMediaStream = TRUE; }
    BOOL m_HasStartedMediaStream;
#endif

    BOOL m_HasReceivedSDP;
    BOOL m_EnableLocalRefer;
    //int m_OutboundAttempts;
    static OString GenerateCallIdExtension();

  public:
    PINLINE void SetRewriteToURI( BOOL rewrite = TRUE ){ m_RewriteToURI = rewrite; };
    PINLINE BOOL WillRewriteToURI()const{ return m_RewriteToURI; };
    PINLINE void SetSendKeepAlive( BOOL send = TRUE ) { m_SendSessionKeepAlive = send; };
    PINLINE BOOL WillSendKeepAlive()const{ return m_SendSessionKeepAlive; };
  protected:
    BOOL m_RewriteToURI;
    BOOL m_ConnectionDestroyed;
    BOOL m_SendSessionKeepAlive;
    
/// IVR related members
  public:
    BOOL AttachIVRContext( IVRContext * );
    
    virtual void OnAttachIVRContext( 
      IVRContext * context 
    );

    BOOL DetachIVRContext();

    virtual void OnDetachIVRContext();

    BOOL IVRPlayText(
      const OString & text 
    );

    BOOL IVRPlayFile(
      const OString & identifier,
      const OString & fn 
    );

    BOOL IVRPlayData(
      const OString & identifier,
      const PBYTEArray & data
    );

    BOOL IVRPlayResource(
      const OString & identifier,
      const PURL & url
    );

    BOOL IVRPlaySilence(
      PINDEX msecs = 0
    );

    BOOL IVRPlaySilence(
      const PTimeInterval & timeout
    );

    BOOL IVRStartRecording(
      const PFilePath & fn, BOOL recordDTMFTerm, 
      const PTimeInterval & recordMaxTime, 
      const PTimeInterval & recordFinalSilence
    );

    BOOL IVRSetDTMFGrammar(
        const OString & identifier,
        int maxDigits, /// maximum digitd collected
        char termChar, /// terminating char to end collection
        BOOL canStopPlayBack = TRUE,
        const PTimeInterval & collectTimeout = 30000,
        const PTimeInterval & perDigitTimeout = PMaxTimeInterval
    );

    BOOL IVRTransferCall(
      const OString & destNumber,
	  BOOL destroyConnectionOnReject
    );

    BOOL IVRTransferCall(
      SIPURI & uri,
	  BOOL destroyConnectionOnReject
    );

    virtual BOOL IVROnOpenChannel(
      IVRContext * context 
    );

    virtual void IVROnEndPlaying( 
      const OString & identifier 
    );

    virtual BOOL IVROnUserInput(
      const OString & str
    );

    virtual void IVROnEndCollection( 
      const OString &grammarId, 
      const OString &grammarBuffer 
    );

	  virtual void IVROnTransferReject(
	    const SIPMessage & reject
	  );

    BOOL IVREndRecording();
    
    BOOL IVRIsPlaying() const;
    
    BOOL IVRIsRecording() const;

    PTextToSpeech * IVRSetTextToSpeech(PTextToSpeech * _tts, BOOL autoDelete = FALSE);
    PTextToSpeech * IVRSetTextToSpeech(const OString & ttsName);
    PTextToSpeech * IVRGetTextToSpeech();

    PINLINE BOOL IsIVRTransfer()const{ return m_IsIVRTransfer; };

  protected:
    PMutex m_IVRContextMutex;
    IVRContext * m_IVRContext;
    BOOL m_IsIVRTransfer;
	BOOL m_IVRDestroyConnectionOnReject;

    
  };
}
    
#endif

