/*
 *
 * Contact.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: Contact.h,v $
 * Revision 1.7  2007/07/14 07:50:21  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.6  2007/02/19 08:48:59  joegenbaclor
 * More work on privacy.
 * 1.  Change PAssertedIdentity base from SIPHeaderF to Contact to allow displaynames
 * 2.  Removed unused variables in ProfileUA
 * 3.  Allowed Privacy to be enabled via Makecall
 * 4.  Improved SIP session privacy support
 *
 * Revision 1.5  2007/02/06 16:50:03  joegenbaclor
 * 1.  More work on TCP
 * 2.  Mutexed SIPHeadeF and SIPHeaderC collection members
 * 3.  Correct bug in B2BUACall record routing
 * 4.  Added Strict Router support to SIPSession::SendRequest
 *
 * Revision 1.4  2006/11/30 08:19:36  joegenbaclor
 * Relicensed opensipstack to MPL/GPL/LGPL triple license scheme
 *
 * Revision 1.3  2006/03/14 03:13:21  joegenbaclor
 * Initial header upload for BETA Release Candidate
 *
 *
 */

#ifndef CONTACT_H
#define CONTACT_H

#include "SIPHeaderC.h"
#include "MimeParam.h"
#include "ContactURI.h"

//Contact: "Mr. Watson" <sip:watson@worcester.bell-telephone.com>
//         ;q=0.7; expires=3600,
//         "Mr. Watson" <mailto:watson@bell-telephone.com> ;q=0.1

namespace SIPParser
{
  class Contact : public SIPHeaderC
  {
    PCLASSINFO( Contact, SIPHeaderC );
  public:
      
    Contact();

    Contact( 
      const Contact & header 
    );
      
    Contact( 
      const OString & header 
    );
      
    Contact( 
      const MimeHeader & header 
    );
      
    Contact( 
      const OStringArray & header 
    );

    Contact(
      const ContactURI::Collection & uri
    );

    Contact & operator=(
      const ContactURI::Collection & uri
    );

    virtual PObject * Clone()const;

    BOOL GetURI(
      ContactURI & uri,
      PINDEX uriIndex = 0
    );

    BOOL AddURI(
      const ContactURI & uri
    );

    BOOL SetURI(
      const ContactURI & uri,
      PINDEX uriIndex = 0
    );
   
  };

  class ContactList : public SIPHeaderCList
  {
    PCLASSINFO( ContactList, SIPHeaderCList );
  public:

    ContactList(){};

    ContactList(
      const SIPHeaderCList & list
    ) : SIPHeaderCList( list ){};

    ContactList(
      const SIPHeaderC::Collection & list
    ) : SIPHeaderCList( list ){};
  };

/// privacy headers
  class PAssertedIdentity : public Contact
  {
    PCLASSINFO( PAssertedIdentity , Contact);
  public:
      
    PAssertedIdentity ();

    PAssertedIdentity ( 
      const PAssertedIdentity & header 
    );
      
    PAssertedIdentity ( 
      const OString & header 
    );
      
    PAssertedIdentity ( 
      const MimeHeader & header 
    );
      
    PAssertedIdentity ( 
      const OStringArray & header 
    );

    PAssertedIdentity (
      const ContactURI::Collection & uri
    );
  };


  class PPreferredIdentity : public Contact
  {
    PCLASSINFO( PPreferredIdentity , Contact);
  public:
      
    PPreferredIdentity ();

    PPreferredIdentity ( 
      const PPreferredIdentity & header 
    );
      
    PPreferredIdentity ( 
      const OString & header 
    );
      
    PPreferredIdentity ( 
      const MimeHeader & header 
    );
      
    PPreferredIdentity ( 
      const OStringArray & header 
    );

    PPreferredIdentity (
      const ContactURI::Collection & uri
    );

  };
};

#endif


