
/*
 *
 * Encryption.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: Encryption.h,v $
 * Revision 1.2  2007/07/14 07:50:21  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.1  2007/04/10 06:13:07  joegenbaclor
 * Added new Encryption Mode
 *
 *
 */

#ifndef ENCRYPTION_H
#define ENCRYPTION_H


#include <ptlib.h>
#include "OString.h"

using namespace Tools;


namespace Encryption
{

  class Engine
  {
  public:
    enum Type
    {
      Type_XOR,
      Type_DiffShift,
      NumType
    };

    static Type m_Type;
    static OString m_Key;
    static OString m_DefKey;

    static BOOL Encrypt(
      const PBYTEArray & in,
      PBYTEArray & out
    );

    static BOOL Encrypt(
      const PBYTEArray & in,
      OString & out
    );

    static BOOL Decrypt(
      const PBYTEArray & in,
      PBYTEArray & out
    );

    static BOOL Decrypt(
      const PBYTEArray & in,
      OString & out
    );
    
    class XOR
    {
    public:
      static BOOL Encrypt(
        const PBYTEArray & in,
        PBYTEArray & out,
        const PBYTEArray & key
      );

      static BOOL Encrypt(
        const PBYTEArray & in,
        OString & out,
        const PBYTEArray & key
      );

      static BOOL Encrypt(
        const PBYTEArray & in,
        PBYTEArray & out
      );

      static BOOL Encrypt(
        const PBYTEArray & in,
        OString & out
      );

      static PBYTEArray m_Key;

    };

    class DiffShift
    {
    public:
      static BOOL Encrypt(
        const PBYTEArray & in,
        PBYTEArray & out,
        const PBYTEArray & key
      );

      static BOOL Encrypt(
        const PBYTEArray & in,
        OString & out,
        const PBYTEArray & key
      );

      static BOOL Decrypt(
        const PBYTEArray & in,
        PBYTEArray & out,
        const PBYTEArray & key
      );

      static BOOL Decrypt(
        const PBYTEArray & in,
        OString & out,
        const PBYTEArray & key
      );

      static BOOL Encrypt(
        const PBYTEArray & in,
        PBYTEArray & out
      );

      static BOOL Encrypt(
        const PBYTEArray & in,
        OString & out
      );

      static BOOL Decrypt(
        const PBYTEArray & in,
        PBYTEArray & out
      );

      static BOOL Decrypt(
        const PBYTEArray & in,
        OString & out
      );


      static PBYTEArray m_Key;
    }; 
  };
}

#endif


