/*
 *
 * MediaServer.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: MediaServer.h,v $
 * Revision 1.6  2007/07/14 07:50:21  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.5  2007/06/05 16:27:54  joegenbaclor
 * Added IVR audio playback functions to B2BUAConnection
 *
 * Revision 1.4  2007/06/04 16:05:31  joegenbaclor
 * *** empty log message ***
 *
 * Revision 1.3  2007/05/29 17:42:33  joegenbaclor
 * Initial work on auto attendant
 *
 * Revision 1.2  2007/03/08 03:36:45  joegenbaclor
 * Transferred declarration of allcodecs.h in transacoders.cxx to avoid multiple definition
 *
 * Revision 1.1  2007/03/07 23:20:21  joegenbaclor
 * Added media server files
 *
 *
 */

#ifndef MEDIASERVER_H
#define MEDIASERVER_H

#include "OpalOSSEndPoint.h"
#include "OpalOSSConnection.h"
#include <opal/ivr.h>

using namespace OPAL;


#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4510)
#pragma warning(disable:4610)
#endif
#include <list>
#if defined(_MSC_VER)
#pragma warning(pop)
#endif

namespace MS
{
  class MediaServerConnection;
  class MediaServerEndPoint;

  class MediaServer : public OpalManager, public Logger
  {
    PCLASSINFO( MediaServer, OpalManager );
    public:

    class CodecList
    {
    public:
      CodecList(
        const char * source,
        const PString & format
      ) : sourceProtocol(source),
          mediaFormat(format),
          preferenceOrder(-1), // -1 indicates disabled
          dirty(false)
      { }

      BOOL operator<(const CodecList & other) { return preferenceOrder < other.preferenceOrder; }

      const char    * sourceProtocol;
      OpalMediaFormat mediaFormat;
      int             preferenceOrder;
      BOOL            dirty;
    };

    typedef std::list<CodecList> MediaInfo;

    MediaServer( 
      CallSessionManager * b2bUA 
    );

    virtual ~MediaServer();

    virtual BOOL Initialize();

    void InitMediaInfo(
      const char * source, 
      const OpalMediaFormatList & formats
    );
    
    void ApplyMediaInfo();
    
    OpalIVRConnection * CreateConnection(
      OpalCall & call,
      const PString & token,
      void * userData,
      const PString & vxml
    );

    virtual MediaServerEndPoint * CreateMediaServerEP();
    
    virtual OpalOSSEndPoint * CreateSIPEP();

    CallSessionManager * GetB2BUA()const;

    PINLINE const OString & GetInterfaceAddress()const{ return m_InterfaceAddress; };
    PINLINE void SetInterfaceAddress( const OString & iface ){ m_InterfaceAddress = iface; };
    PINLINE WORD GetListenerPortBase()const{ return m_ListenerPortBase; };
    PINLINE void SetListenerPortBase( WORD port ){ m_ListenerPortBase = port; };
    PINLINE PIPSocket::Address GetListenerAddress()const{ return m_ListenerAddress; };
    PINLINE WORD GetListenerPort()const{ return m_ListenerPort; };
    PINLINE MediaServerEndPoint * GetMediaServerEndPoint()const{ return m_MediaServerEP; };
    
  protected:
    MediaServerEndPoint * m_MediaServerEP;
    OpalOSSEndPoint * m_SIPEP;
    MediaInfo m_MediaInfo;
    PStringArray m_MediaFormats;
    OString m_InterfaceAddress;
    WORD m_ListenerPortBase;
    BOOL m_IsInitialized;
    PIPSocket::Address m_ListenerAddress;
    WORD m_ListenerPort;
    CallSessionManager * m_B2BUA;
  };


  class MediaServerEndPoint : public OpalIVREndPoint
  {
    PCLASSINFO( MediaServerEndPoint, OpalIVREndPoint );
  public:

    MediaServerEndPoint(
      MediaServer & manager
    );

    OpalIVRConnection * CreateConnection(
      OpalCall & call,
      const PString & token,
      void * userData,
      const PString & vxml
    );
  };

}

#endif

