/*
 *
 * MediaServerSession.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: MediaServerSession.h,v $
 * Revision 1.4  2007/06/08 15:10:28  joegenbaclor
 * Added IVR DTMF Grammar
 *
 * Revision 1.3  2007/06/08 02:33:45  joegenbaclor
 * Commiting more Media Server Related Changes
 *
 * Revision 1.2  2007/06/05 16:27:54  joegenbaclor
 * Added IVR audio playback functions to B2BUAConnection
 *
 * Revision 1.1  2007/06/04 16:05:31  joegenbaclor
 * *** empty log message ***
 *
 *
 */

#ifndef MEDIASERVERSESSION_H
#define MEDIASERVERSESSION_H

#include "MediaServer.h"
#include "SIPSession.h"

namespace MS
{
  class MediaServerSession;
  class IVRDTMFGrammar;
  class IVRContext : public OpalVXMLSession 
  {
    PCLASSINFO(IVRContext, OpalVXMLSession);
    public:
      IVRContext(
        MediaServerSession * _conn,
        PTextToSpeech * tts = NULL,
        BOOL autoDelete = FALSE
      );

      virtual BOOL Open(
        const PString & mediaFormat
      );

      virtual BOOL Close();

      virtual PWAVFile * CreateWAVFile(
        const PFilePath & fn,
        PFile::OpenMode mode,
        int opts,
        unsigned fmt
      );

      BOOL AttachB2BUAConnection(
        SIPSession * session
      );

      MediaServerSession * GetSession()const;

      BOOL SetDTMFGrammar(
        const PString & identifier,
        int maxDigits, /// maximum digitd collected
        char termChar, /// terminating char to end collection
        BOOL canStopPlayBack = TRUE,
        const PTimeInterval & collectTimeout = 30000,
        const PTimeInterval & perDigitTimeout = PMaxTimeInterval
      );

      void OnEndCollection(
        const PString & grammarId,
        const PString & grammarBuffer
      );

      void FlushQueue();

      /// virtuals from PVXMLSession
      virtual void OnEndPlay( const PString & identifier );
      virtual BOOL OnUserInput(const PString & str);

  protected:
    MediaServerSession * m_Session;
    SIPSession::GCRef m_B2BUAConnection;
    PMutex m_GrammarMutex;
    IVRDTMFGrammar * m_CurrentGrammar;
  };

  
  class IVRDTMFGrammar : public PObject
  {
    PCLASSINFO( IVRDTMFGrammar, PObject );
  public:
    IVRDTMFGrammar(
      const PString & identifier,
      IVRContext * session,
      int maxDigits, /// maximum digitd collected
      char termChar, /// terminating char to end collection
      BOOL canStopPlayBack = TRUE,
      const PTimeInterval & collectTimeout = 30000,
      const PTimeInterval & perDigitTimeout = PMaxTimeInterval
    );

    void OnDTMFInput( 
      char dtmf
    );

    void OnEndCollection();

    void StopCollecting();

    PDECLARE_NOTIFIER( PTimer, IVRDTMFGrammar, OnCollectTimeout );
    PDECLARE_NOTIFIER( PTimer, IVRDTMFGrammar, OnDigitTimeout );

  protected:
    BOOL m_CanStopPlayback;
    IVRContext * m_IVRContext;
    int m_MaxDigits;
    char m_TermChar;
    BOOL m_CanStopPlayBack;
    PTimer m_CollectTimer;
    PTimer m_DigitTimer;
    PString m_Collected;
    BOOL m_CanCollectDigits;
    PMutex m_DigitsMutex;
    PString m_Identifier;
  };

  class MediaServerSession : public OpalIVRConnection
  {
      PCLASSINFO(MediaServerSession, OpalIVRConnection);
    public:
      MediaServerSession(
        OpalCall & call,            ///<  Owner calll for connection
        MediaServerEndPoint & endpoint, ///<  Owner endpoint for connection
        const PString & token      ///<  Token for connection
      );
      PINLINE void SetIVRContext( IVRContext * ctx ){ vxmlSession = ctx; };
  };
};


#endif


