
/*
 *
 * OString.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: OString.h,v $
 * Revision 1.2  2007/07/16 09:43:55  joegenbaclor
 * PTRACE now uses internal OSS Logger
 *
 * Revision 1.1  2007/07/14 07:50:28  joegenbaclor
 * Migrated code to use new STL based string class
 *
 *
 */

#ifndef OString_H
#define OString_H

#include <ext_string.h>
#include <string>
#include <sstream>
#include <iostream>
#include <ptlib.h>
using namespace std;


namespace Tools
{


  class OStringArray;

	class OString : public ext_string
	{
  public:
      enum Comparison {
        LessThan = -1,
        EqualTo = 0,
        GreaterThan = 1
      };

			OString();

      OString( char c );

      OString(
        const string &s, 
        size_type pos = 0, 
        size_type n = npos
      );

			OString(
        const value_type *s
      );

			OString(
        const value_type *s, 
        size_type n
      );


			OString(
        size_type n, 
        value_type c
      );


      OString(
        const PString & s
      );

      OString(
        const PCaselessString & s
      );

      OString(
        const ostringstream & s
      );

      template < class T >
      OString(const T &arg);

      OString(short n);


      OString(unsigned short n);


      OString(int n);


      OString(unsigned int n);


      OString(long n);


      OString(unsigned long n);


      OString(PInt64 n);


      OString(PUInt64 n);

      template <class InputIterator>
			OString(
        InputIterator first, 
        InputIterator last
      );

      
      template < class T >
      static OString ToString(const T &arg);

      OString Trim()const;

      OString ToLower()const;
      OString ToUpper()const;

      PINDEX Tokenise(
        vector<OString> & tokens,
        const OString & delimiters,
        BOOL includeBlanks = TRUE
      ) const;
      OStringArray Tokenise(
        const OString & delimiters,
        BOOL includeBlanks = TRUE
      ) const;
      
      PINDEX GetSize()const;

      PINDEX GetLength()const;

      OString Left(
        PINDEX len
      )const;
      OString Right(
        PINDEX len
      )const;

      OString Mid(
        PINDEX offset
      )const;

      OString Mid(
        PINDEX offset,
        PINDEX len
      )const;

      long AsInteger(unsigned base = 10) const;


      unsigned long AsUnsigned(unsigned base = 10) const;

      Comparison InternalCompare(
        PINDEX offset, PINDEX length, const char * cstr) const;

      OString::Comparison InternalCompare(PINDEX offset, char c) const;

      PINDEX Find(
        const char * cstr,    ///< C string to search for in string.
        PINDEX offset = 0     ///< Offset into string to begin search.
      ) const;

      PINDEX FindLast(
        const char * cstr    ///< C string to search for in string.
      ) const;

      static OString Empty();

      bool IsEmpty()const;

      BOOL MakeUnique();

      void Replace(
        const OString & target,
        const OString & subs,
        BOOL dummy = TRUE
      );

      PINDEX FindOneOf(const char * cset, PINDEX offset = 0) const;

      OStringArray Lines() const;

      const  char * GetPointer( int dummy );
      OString & operator=( char c );

      BOOL operator *= (
        const OString & ostring
      )const;

      OString * Clone()const;

      PINDEX HashFunction() const;

  };

  //---------------------------------------------------------

  typedef vector<OString> OStringVector;
  typedef ostringstream OStringStream;

  class OStringArray : public OStringVector
  {
  public:
    OStringArray();

    ~OStringArray();

    void AppendString(
      const OString & ostring
    );

    void RemoveAt( PINDEX offset );

    void InsertAt( PINDEX offset,
      const OString & ostring
    );

    PINDEX GetSize()const;

    OStringArray * Clone()const;

    BOOL IsEmpty()const;
    


  };

}
#endif
