/*
 *
 * ResourceCounter.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: ResourceCounter.h,v $
 * Revision 1.4  2007/07/15 07:49:57  joegenbaclor
 * tidying up resource counters
 *
 * Revision 1.3  2007/07/10 03:23:02  joegenbaclor
 * Added experimental code to use STL tokenizer instead of internal string tokenizer
 *
 * Revision 1.2  2007/07/09 03:53:53  joegenbaclor
 * Added ResourceCounter to Tools namespace
 * Updated VC8 project files
 *
 * Revision 1.1  2007/07/09 03:25:22  joegenbaclor
 * Added resource counter class
 *
 *
 */
 
#ifndef RESOURCE_COUNTER_H
#define RESOURCE_COUNTER_H

#include <ptlib.h>

namespace Tools
{

class ResourceCounter
{
  public:
    
    // Object counters
    static PAtomicInteger m_RC_ICT;
    static PAtomicInteger m_RC_NICT;
    static PAtomicInteger m_RC_IST;
    static PAtomicInteger m_RC_NIST;
    static PAtomicInteger m_RC_CallSession;
    static PAtomicInteger m_RC_Connection;
    static PAtomicInteger m_RC_Registration;
    static PAtomicInteger m_RC_RTPSession;
    static PAtomicInteger m_RC_EventQueue;
    static PAtomicInteger m_RC_Cache;
    static PAtomicInteger m_RC_Garbage;
    
    // maximum value
    static PAtomicInteger m_RC_MaxICT;
    static PAtomicInteger m_RC_MaxNICT;
    static PAtomicInteger m_RC_MaxIST;
    static PAtomicInteger m_RC_MaxNIST;
    static PAtomicInteger m_RC_MaxCallSession;
    static PAtomicInteger m_RC_MaxConnection;
    static PAtomicInteger m_RC_MaxRegistration;
    static PAtomicInteger m_RC_MaxRTPSession;
    static PAtomicInteger m_RC_MaxEventQueue;
    static PAtomicInteger m_RC_MaxCache;
    static PAtomicInteger m_RC_MaxGarbage;
    
    // high value
    static PAtomicInteger m_RC_HighICT;
    static PAtomicInteger m_RC_HighNICT;
    static PAtomicInteger m_RC_HighIST;
    static PAtomicInteger m_RC_HighNIST;
    static PAtomicInteger m_RC_HighCallSession;
    static PAtomicInteger m_RC_HighConnection;
    static PAtomicInteger m_RC_HighRegistration;
    static PAtomicInteger m_RC_HighRTPSession;
    static PAtomicInteger m_RC_HighEventQueue;
    static PAtomicInteger m_RC_HighCache;
    static PAtomicInteger m_RC_HighGarbage;
    
    static int IncrementICT();
    static int DecrementICT();
    static int IncrementNICT();
    static int DecrementNICT();
    static int IncrementIST();
    static int DecrementIST();
    static int IncrementNIST();
    static int DecrementNIST();
    static int IncrementCallSession();
    static int DecrementCallSession();
    static int IncrementConnection();
    static int DecrementConnection();
    static int IncrementRegistration();
    static int DecrementRegistration();
    static int IncrementRTPSession();
    static int DecrementRTPSession();
    static int IncrementEventQueue();
    static int DecrementEventQueue();
    static int IncrementCache();
    static int DecrementCache();
    static int IncrementGarbage();
    static int DecrementGarbage();
    
    static BOOL IsICTMaxReached();
    static BOOL IsNICTMaxReached();
    static BOOL IsISTMaxReached();
    static BOOL IsNISTMaxReached();
    static BOOL IsCallSessionMaxReached();
    static BOOL IsConnectionMaxReached();
    static BOOL IsRegistrationMaxReached();
    static BOOL IsRTPSessionMaxReached();
    static BOOL IsEventQueueMaxReached();
    static BOOL IsCacheMaxReached();
    static BOOL IsGarbageMaxReached();
    
    static void Print( ostream & strm );

};

}

#endif







