/*
 *
 * RouteURI.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: RouteURI.h,v $
 * Revision 1.6  2007/07/14 07:50:27  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.5  2007/02/06 16:50:04  joegenbaclor
 * 1.  More work on TCP
 * 2.  Mutexed SIPHeadeF and SIPHeaderC collection members
 * 3.  Correct bug in B2BUACall record routing
 * 4.  Added Strict Router support to SIPSession::SendRequest
 *
 * Revision 1.4  2006/11/30 08:19:36  joegenbaclor
 * Relicensed opensipstack to MPL/GPL/LGPL triple license scheme
 *
 * Revision 1.3  2006/03/14 03:13:21  joegenbaclor
 * Initial header upload for BETA Release Candidate
 *
 *
 */

#ifndef RouteURI_H
#define RouteURI_H

#include <ptlib.h>
#include "SIPURI.h"

namespace SIPParser
{
  class RouteURI : public PObject
  {
    PCLASSINFO( RouteURI, PObject );
  public:

    RouteURI();

    RouteURI(
      const RouteURI & uri 
    );

    RouteURI(
      const OString & uri 
    );

    RouteURI(
      const SIPURI & uri,
      BOOL isLooseRouter = TRUE
    );

    RouteURI & operator=(
      const OString & uri
    );

    RouteURI & operator=(
      const SIPURI & uri
    );

    RouteURI & operator=(
      const RouteURI & uri
    );

    virtual void PrintOn( 
      ostream & strm
    )const;

    virtual PObject * Clone()const;

    PINLINE BOOL IsLooseRouter()const{ return m_IsLr; };

    PINLINE void SetLooseRouter( BOOL isLr = FALSE ){ m_IsLr = isLr; };
    
    PINLINE const SIPURI & GetURI()const{ return m_URI; };

    PINLINE void SetURI( SIPURI & uri ){ m_URI = uri; };

    PLIST( Collection, RouteURI );

  protected:

    BOOL m_IsLr;
    SIPURI m_URI;
  };
};

#endif

