/*
 *
 * SDPLazyParser.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: SDPLazyParser.h,v $
 * Revision 1.7  2007/07/14 07:50:27  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.6  2007/02/11 14:41:59  joegenbaclor
 * 1.  Added capability to add SDP attributes
 * 2.  Appned a=nortpproxy:yes to SDP translation
 *
 * Revision 1.5  2007/02/11 10:44:35  joegenbaclor
 * Added SetOriginAddress when translating SDP
 *
 * Revision 1.4  2006/12/08 09:54:26  joegenbaclor
 * Added ability to get Media direction
 *
 * Revision 1.3  2006/11/30 08:19:36  joegenbaclor
 * Relicensed opensipstack to MPL/GPL/LGPL triple license scheme
 *
 * Revision 1.2  2006/06/14 10:15:45  joegenbaclor
 * Added SetMediaAddress method
 *
 * Revision 1.1  2006/06/14 08:43:38  joegenbaclor
 * Initial upload of OpenB2BUA applcation and related classes
 *
 *
 */

#ifndef SDPLAZYPARSER_H
#define SDPLAZYPARSER_H

#include <ptlib.h>
#include "SDPHeader.h"
#include "SIPURI.h"

using namespace SIPParser;

namespace SDP
{
  class SDPLazyParser : public OString
  {
  public:

    enum MediaType
    {
      Audio,
      Video,
      Image,
      NumMediaType
    };

    enum MediaDirection
    {
      SendOnly,
      ReceiveOnly,
      SendAndReceive,
      Inactive,
      NumMediaDescription
    };

    SDPLazyParser();

    SDPLazyParser(
      const OString & sdp
    );

    SDPLazyParser(
      const SDPLazyParser & sdp
    );

    SDPLazyParser & operator=( 
      const OString & sdp 
    );

    SDPLazyParser & operator=( 
      const SDPLazyParser & sdp 
    );

    BOOL GetMediaAddress(
      MediaType type,
      SIPURI & address,
      PINDEX offSet = 0
    )const;

    PINDEX GetMediaCount(
      MediaType type
    )const;

    MediaDirection GetMediaDirection(
      MediaType type,
      PINDEX offSet = 0
    )const;

    BOOL SetMediaDirection(
      MediaDirection direction,
      MediaType type,
      PINDEX offSet = 0
    );

    BOOL GetMediaFormat(
      MediaType type,
      PINDEX mediaOffSet,
      PINDEX formatOffSet,
      int & payloadType,
      OString & formatName,
      SIPURI & address
    )const;

    PINDEX GetMediaFormatCount(
      MediaType type,
      PINDEX offSet = 0
    )const;

    BOOL SetMediaAddress(
      MediaType type,
      PINDEX mediaOffSet,
      const OString & host,
      WORD port
    );

    BOOL AppendMediaAttribute(
      MediaType type,
      PINDEX mediaOffSet,
      const OString & attribute
    );

    OString GetOriginAddress()const;

    BOOL SetOriginAddress(
      const OString & address
    );

    BOOL Read(
      PChannel & inputStream
    );
  };
};


#endif

