/*
 *
 * SIPTCPTransport.h
 * 
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: SIPTCPTransport.h,v $
 * Revision 1.11  2007/07/14 07:50:27  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.10  2007/02/27 08:18:33  joegenbaclor
 * Introduces new stages for inbound and outbound processing
 *
 * Revision 1.9  2007/02/27 02:11:26  joegenbaclor
 * *** empty log message ***
 *
 * Revision 1.8  2007/02/22 14:40:29  joegenbaclor
 * Completed multi listener support
 *
 * Revision 1.7  2007/01/29 08:35:56  joegenbaclor
 * More work on TCP Transport
 *
 * Revision 1.6  2007/01/24 10:19:54  joegenbaclor
 * Work on TCP support has commenced.
 *
 * Revision 1.5  2006/11/30 08:19:37  joegenbaclor
 * Relicensed opensipstack to MPL/GPL/LGPL triple license scheme
 *
 * Revision 1.4  2006/06/28 14:14:57  joegenbaclor
 * Bulk commit for Media Proxy functionalities for OpenB2BUA
 * Marks a minor version upgrade from 1.0.2 to 1.0.3
 *
 * Revision 1.3  2006/03/14 03:13:21  joegenbaclor
 * Initial header upload for BETA Release Candidate
 *
 *
 */

#ifndef SIPTCPTRANSPORT_H
#define SIPTCPTRANSPORT_H

#include "SIPTransport.h"

namespace SIPTransports
{
  class SIPTransportManager;

  class SIPTCPTransport : public SIPTransport
  {
    PCLASSINFO( SIPTCPTransport, SIPTransport );
  public:

    SIPTCPTransport(
      SIPTransportManager * manager
    );

    ~SIPTCPTransport();
        
    BOOL ReadPacket(
      SIPPacket & packet
    );

    BOOL ReadPacket( 
      SIPMessage *& packet,
      PIPSocket::Address & addr,
      WORD & port,
      PIPSocket::Address & ifaceAddress,
      WORD & ifacePort,
      int & length
    );

    BOOL WritePacket( 
      SIPMessage & packet,
      const PIPSocket::Address & address,
      WORD port,
      const OString & iface
    );

    BOOL WriteString(
      const OString & packet,
      const PIPSocket::Address & sendAddress,
      WORD sendPort,
      const OString & iface = OString::Empty(),
      WORD localPort = 0
    );

    virtual BOOL WriteBytes(
      PBYTEArray & packet,
      const PIPSocket::Address & sendAddress,
      WORD sendPort,
      const OString & iface = OString::Empty(),
      WORD localPort = 0
    );

    BOOL ReadPacket( 
      SIPMessage *& packet,
      PIPSocket::Address & addr,
      WORD & port,
      int & length
    );

    BOOL WritePacket( 
      SIPMessage & packet,
      const PIPSocket::Address & address,
      WORD port
    );

    BOOL Listen(
      const OString & iface = OString::Empty(),
      WORD port = 5060
    );

    BOOL Listen(
      const OString & iface,
      WORD port,
      BOOL append
    );


    BOOL GetListenerAddress(
      const PIPSocket::Address & target,
      PIPSocket::Address & listenerAddress,
      WORD & listenerPort
    );

    BOOL IsLocalAddressAndPort(
      const PIPSocket::Address & addr,
      WORD port
    )const;

    virtual void CleanUp();

  protected:
    PTCPSocket  * m_ListenerSocket;
   
  };
};

#endif


