/*
 *
 * VXMLInterface.h
 *
 * Open SIP Stack ( OSS )
 *
 * Copyright (C) 2006 Joegen E. Baclor. All Rights Reserved.
 *
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is OpenSIPStack Library.
 *
 * The Initial Developer of the Original Code is Joegen E. Baclor.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * $Log: VXMLInterface.h,v $
 * Revision 1.3  2007/07/14 07:50:28  joegenbaclor
 * Migrated code to use new STL based string class
 *
 * Revision 1.2  2007/05/03 10:35:08  joegenbaclor
 * more class definitions
 *
 * Revision 1.1  2007/05/02 14:04:56  joegenbaclor
 * Initial upload
 *
 *
 */

#ifndef VXMLINTERFACE_H
#define VXMLINTERFACE_H

#include <ptlib.h>
#include <ptclib/pxml.h>
#include <ptclib/delaychan.h>
#include <ptclib/pwavfile.h>
#include <ptclib/ptts.h>
#include <ptclib/url.h>

#include "OString.h"
using namespace Tools;

namespace VXML
{
  class VXMLInterface : public PObject
  {
    PCLASSINFO( VXMLInterface, PObject );
  public:
    virtual OString VXMLGetApplicationName();
    virtual OString VXMLGetAppVersion();
    virtual OString VXMLGetInterfaceName() = 0;
  };

  class VXMLElement : public PObject
  {
    PCLASSINFO( VXMLElement, PObject );
  public:
    VXMLElement();

    virtual ~VXMLElement();

    virtual BOOL ProcessEvents( 
      VXMLInterface * iface 
    ) = 0;

    virtual void PrintOn( ostream & strm )const;

    PINLINE VXMLElement & operator=( PXMLElement * block ){ m_XML = block; return *this; }; 
    PINLINE void SetXML( PXMLElement * block ){ m_XML = block; };
    PINLINE const PXMLElement * GetXML()const{ return m_XML; };

  protected:
    PXMLElement * m_XML;

  };
};

#endif


